@@ -83,8 +83,8 @@ class _PK1AsyncMethodCall:
                         reply_handler=self._pk_reply_handler,
                         error_handler=self._pk_error_handler,
                         timeout=3600)
-        except TypeError, e:
-            debugprint ("Type error in PK call: %s" % e)
+        except TypeError as e:
+            debugprint ("Type error in PK call: %s" % repr (e))
             self.call_fallback_fn ()
 
     def _destroy (self):
@@ -120,7 +120,7 @@ class _PK1AsyncMethodCall:
             self._destroy ()
             return
 
-        debugprint ("PolicyKit method failed with: %s" % repr (str (error)))
+        debugprint ("PolicyKit method failed with: %s" % repr (error))
         self.call_fallback_fn ()
 
     def _pk_error_handler (self, exc):
@@ -144,7 +144,7 @@ class _PK1AsyncMethodCall:
             return
 
         debugprint ("PolicyKit call to %s did not work: %s" %
-                    (self._pk_method_name, exc))
+                    (self._pk_method_name, repr (exc)))
         self.call_fallback_fn ()
 
     def call_fallback_fn (self):
@@ -269,8 +269,8 @@ class PK1Connection:
 
                     break
 
-            except Exception, e:
-                debugprint ("Exception assessing DevicesGet API: %s" % e)
+            except Exception as e:
+                debugprint ("Exception assessing DevicesGet API: %s" % repr (e))
 
         methodtype = type (self._conn.getPrinters)
         bindings = []
@@ -341,7 +341,7 @@ class PK1Connection:
                 # ignored.  Skip to the next one.
                 argindex += 1
                 continue
-            except TypeError, e:
+            except TypeError as e:
                 debugprint ("Error converting %s to %s" %
                             (repr (arg), types[argindex]))
                 return result
@@ -353,7 +353,7 @@ class PK1Connection:
             if leftover_kwds.has_key (kw):
                 try:
                     val = self._coerce (types[argindex], leftover_kwds[kw])
-                except TypeError, e:
+                except TypeError as e:
                     debugprint ("Error converting %s to %s" %
                                 (repr (leftover_kwds[kw]), types[argindex]))
                     return result
@@ -388,7 +388,7 @@ class PK1Connection:
             try:
                 debugprint ("Calling PK method %s" % pk_method_name)
                 asyncmethodcall.call ()
-            except dbus.DBusException, e:
+            except dbus.DBusException as e:
                 debugprint ("D-Bus call failed: %s" % repr (e))
                 use_pycups = True
 
@@ -439,7 +439,7 @@ class PK1Connection:
         result_str = dict()
         for key, value in dbusdict.iteritems ():
             if type (key) == dbus.String:
-                result_str[str (key)] = str (value)
+                result_str[unicode (key)] = unicode (value)
             else:
                 result_str[key] = value
 
