@@ -247,6 +247,8 @@ class Connection:
                 break
             except cups.IPPError as e:
                 (e, m) = e.args
+                if isinstance(m, bytes):
+                    m = m.decode('utf-8', 'replace')
                 if self._use_pk and m == 'pkcancel':
                     raise cups.IPPError (0, _("Operation canceled"))
 
@@ -474,9 +476,9 @@ class Connection:
         else:
             d = AuthDialog (parent=self._parent)
 
-        d.set_prompt (self._prompt)
-        d.set_auth_info ([self._use_user, ''])
-        d.field_grab_focus ('password')
+        d.set_prompt ('')
+        d.set_auth_info (['', ''])
+        d.field_grab_focus ('username')
         d.set_keep_above (True)
         d.show_all ()
         d.show_now ()
