@@ -962,7 +962,19 @@ class JobViewer (GtkGUI):
                     if result == GnomeKeyring.Result.OK:
                         auth_info = map (lambda x: '', auth_info_required)
                         ind = auth_info_required.index ('username')
-                        auth_info[ind] = items[0].attributes.get ('user', '')
+
+                        for attr in GnomeKeyring.attribute_list_to_glist (
+                                items[0].attributes):
+                            # It might be safe to assume here that the
+                            # user element is always the second item in a
+                            # NETWORK_PASSWORD element but lets make sure.
+                            if attr.name == 'user':
+                                auth_info[ind] = attr.get_string()
+                                break
+                        else:
+                            debugprint ("Did not find username keyring "
+                                        "attributes.")
+
                         ind = auth_info_required.index ('password')
                         auth_info[ind] = items[0].secret
                     else:
@@ -995,7 +1007,7 @@ class JobViewer (GtkGUI):
                         c._end_operation ()
                         nonfatalException ()
 
-                if data.has_key ('auth_info_required'):
+                if data.has_key ('auth-info-required'):
                     username = pwd.getpwuid (os.getuid ())[0]
                     keyring_attrs["user"] = str (username)
                     self.display_auth_info_dialog (job, keyring_attrs)
@@ -1077,7 +1089,7 @@ class JobViewer (GtkGUI):
 
         if remember:
             try:
-                keyring = GnomeKeyring.get_default_keyring_sync ()
+                (result, keyring) = GnomeKeyring.get_default_keyring_sync ()
                 type = GnomeKeyring.ItemType.NETWORK_PASSWORD
                 keyring_attrs = getattr (dialog,
                                          "keyring_attrs",
@@ -1102,10 +1114,13 @@ class JobViewer (GtkGUI):
                         GnomeKeyring.Attribute.list_append_string (attrs,
                                                                    key,
                                                                    val)
-                    id = gnomekeyring.item_create_sync (keyring, type, name,
-                                                        attrs, secret, True)
-                    debugprint ("keyring: created id %d for %s" % (repr (id),
-                                                                   name))
+                    (result, id) = GnomeKeyring.item_create_sync (keyring,
+                                                                  type,
+                                                                  name,
+                                                                  attrs,
+                                                                  secret,
+                                                                  True)
+                    debugprint ("keyring: created id %d for %s" % (id, name))
             except:
                 nonfatalException ()
 
@@ -2314,7 +2329,7 @@ class JobViewer (GtkGUI):
                                   GdkPixbuf.InterpType.NEAREST, 255)
                 icon = copy
             except GObject.GError:
-                debugprint ("No %s icon available" % gtk.STOCK_MEDIA_PAUSE)
+                debugprint ("No %s icon available" % Gtk.STOCK_MEDIA_PAUSE)
         else:
             # Check state reasons.
             printer = data['job-printer-name']
